/*
 * Generate a continuous tone on the soundcard
 *
 * (c)2001 Paolo Cravero <ik1zyw at yahoo! dot com>
 */

/* 
 * Standard includes
 */

#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include "tone-encoder.h"


/*
 * Definitions for the tone generator
 */

#define M_PI        3.14159265358979323846
#define volume 100   // strength of the sinusoid, 0-127 accepted
#define MY_FREQ 800  // frequency to monitor, in Hz



int main(int argc, char *argv[]) {

  int buf;  // just a counter
  int len;  // length of bytes read from device
  float buf_time; // time length of the buffer
  unsigned char * audio_buffer; // buffer for reading from soundcard
  float * fft_buffer;  // buffer for computing FFT
  float my_f_m;  // modulus of the searched frequency component
  unsigned int buf_size, sample_rate, my_freq;
  float offset_abs, offset; // offset for sinewave generation

  // check calling parameters
  if (argc<2) {
    printf("\nUsage:\ntone_decoder [sampling_frequency [searched_tone]]\n");
    printf("\nNo parameters given. Continuing with default.\n\n");
  }
  if (argc>=2) {  // read desired sampling frequency
    sample_rate = atoi(argv[1]);
  } else {
    sample_rate = SAMPLE_RATE;
  }
  if (argc>=3) {
    my_freq = atoi(argv[2]);
  } else {
    my_freq=MY_FREQ;
  }
  printf("\nGenerating %d tone.\n",my_freq);

  open_audio();  // open the sound card device for input

  if (DEBUG_CARD_INFO) {
    check_audio_formats();  // let's check available audio formats
    check_audio_features(); // let's check available card features
  }

  set_audio_format();
  set_stereo_mode();
  sample_rate=set_sample_rate(sample_rate);
  buf_size=check_buffer_size(); // read the driver self-computed buffer size and...

  // ...allocate buffer dynamically
  audio_buffer=(unsigned char *)malloc(buf_size*sizeof(unsigned char));

  offset=0;

  while (1) {

    // senerate sinusoid
    for (buf=0;buf<buf_size;buf++) {
      audio_buffer[buf]=(char)(volume*sin(2*M_PI*(float)my_freq*(float)buf/(float)sample_rate+offset)+128+offset);
    }

    // compute phase of last generated sample and extract normalized phase offset
    offset_abs=2*M_PI*(float)my_freq*(float)buf/(float)sample_rate+offset;
    offset=offset_abs-floor(offset_abs/2/M_PI)*2*M_PI;

    if ((len = write(audio_fd, audio_buffer, buf_size)) == -1) {
      perror("audio write");
      exit(2);
    }


  } // end while

} // end main
