/*
 * Prorotypes
 */

void open_audio();
void check_audio_features();
void check_audio_formats();
void set_stereo_mode();
void set_audio_format();
int set_sample_rate(int);
int check_buffer_size();

/*
 * Mandatory variables.
 */

#define DEVICE_NAME   "/dev/dsp"
#define OPEN_MODE   O_WRONLY  // don't touch :)
#define AUDIO_FORMAT  AFMT_U8
#define STEREO   0  // 0 = mono, 1 = stereo
#define SAMPLE_RATE   8192
#define DEBUG_MODE 1 // 0 = no debug info
#define DEBUG_CARD_INFO 1 // 0 = don't print card features


int audio_fd;


void open_audio() {
  // open the audio device
  if ((audio_fd = open(DEVICE_NAME, OPEN_MODE, 0)) == -1) {
   /* Opening device failed */  
    perror(DEVICE_NAME);
    exit(1);
  } // end if audio_fd
  if(DEBUG_MODE) {
    printf ("Audio device successfully opened\n");
  }

  return;
}

void check_audio_formats() {

  int mask;

  printf ("\n* Available audio formats *\n");
  if (ioctl(audio_fd, SNDCTL_DSP_GETFMTS, &mask) == -1) {
    perror("SNDCTL_DSP_GETFMTST");
    exit(5);
  }
  if (mask&AFMT_MU_LAW) {
    printf("AFMT_MU_LAW supported\n");
  }
  if (mask&AFMT_A_LAW) {
    printf("AFMT_A_LAW supported\n");
  }
  if (mask&AFMT_IMA_ADPCM) {
    printf("AFMT_IMA_ADPCM supported\n");
  }
  if (mask&AFMT_U8) {
    printf("AFMT_U8 supported\n");
  }
  if (mask&AFMT_S16_LE) {
    printf("AFMT_S16_LE supported\n");
  }
  if (mask&AFMT_S16_BE) {
    printf("AFMT_S16_BE supported\n");
  }
  if (mask&AFMT_S8) {
    printf("AFMT_S8 supported\n");
  }
  if (mask&AFMT_U16_LE) {
    printf("AFMT_U16_LE supported\n");
  }
  if (mask&AFMT_U16_BE) {
    printf("AFMT_U16_BE supported\n");
  }
  if (mask&AFMT_MPEG) {
    printf("AFMT_MPEG supported\n");
  }
  return;

}

void check_audio_features() {

  // checking available card features
  int caps;

  printf("\n* Available card features *\n");
  if((ioctl(audio_fd, SNDCTL_DSP_GETCAPS, &caps))==-1) {
    perror("SNDCTL_DSP_GETCAPS");
    exit(9);
  }
  if (caps&DSP_CAP_REVISION) {
    printf("DSP_CAP_REVISION supported\n");
  }
  if (caps&DSP_CAP_DUPLEX) {
    printf("DSP_CAP_DUPLEX supported\n");
  }
  if (caps&DSP_CAP_REALTIME) {
    printf("DSP_CAP_REALTIME supported\n");
  }
  if (caps&DSP_CAP_BATCH) {
    printf("DSP_CAP_BATCH supported\n");
  }
  if (caps&DSP_CAP_COPROC) {
    printf("DSP_CAP_COPROC supported\n");
  }
  if (caps&DSP_CAP_TRIGGER) {
    printf("DSP_CAP_TRIGGER supported\n");
  }
  if (caps&DSP_CAP_MMAP) {
    printf("DSP_CAP_MMAP supported\n");
  }
  printf("\n");

  return;
}


void set_audio_format() {

  // let's set the desired format
  int format = AUDIO_FORMAT;

  if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &format)==-1) {
    perror("SNDCTL_DSP_SETFMT");
    exit(3);
  }
  if (format != AUDIO_FORMAT) {
    perror("Format not supported");
    exit(4);
  }

  return;
}


void set_stereo_mode() {

  // setting stereo/mono mode
  int stereo=STEREO;

  if (ioctl(audio_fd, SNDCTL_DSP_STEREO, &stereo)==-1) {
    perror("SNDCTL_DSP_STEREO");
    exit(6);
  }
  if (stereo != STEREO) {
    perror("Device doesn't support stereo setting");
    exit(7);
  }
  if(DEBUG_MODE) {
    printf("Stereo mode succesfully set to %d\n",stereo);
  }
  return;
}


int check_buffer_size() {
  // checking the driver buffer size
  int frag_size;

  if (ioctl(audio_fd, SNDCTL_DSP_GETBLKSIZE, &frag_size) == -1) {
    perror("SNDCTL_DSP_GETBLKSIZE");
    exit(11);
  }
  if (DEBUG_MODE) {
    printf("Driver fragment size is %d\n",frag_size);
  }

  return frag_size;
}


int set_sample_rate(int speed) {

  // now we set the sample rate
  int speed_original=speed;

  if (ioctl(audio_fd, SNDCTL_DSP_SPEED, &speed)==-1) {
    perror("SNDCTL_DSP_SPEED");
    exit(8);
  }
  if (speed!=speed_original) {
  printf("WARNING: sample rate set by sound card to %d\n",speed);
  }

  return speed;
}
